from typing import Optional
from honeyhive.api.models.fine_tuned_models import (
    FineTunedModelResponse,
    ListFineTunedModelResponse,
)
from honeyhive.sdk.init import honeyhive_client


def get_fine_tuned_models(
    project: Optional[str] = None, model_id: Optional[str] = None
) -> ListFineTunedModelResponse:
    """Get all fine tuned models"""
    client = honeyhive_client()
    return client.get_fine_tuned_models(task=project, model_id=model_id)


def get_fine_tuned_model(model_id: str) -> FineTunedModelResponse:
    """Get a fine tuned model"""
    client = honeyhive_client()
    return client.get_fine_tuned_model(model_id=model_id)


__all__ = ["get_fine_tuned_models", "get_fine_tuned_model"]



from typing import Optional, List
from pydantic import BaseModel, Field
import datetime


class MetricResponse(BaseModel):
    id: str = Field(
        title="Metric ID", description="The unique ID of the metric"
    )
    object: str = Field(
        title="Object", description="The object type of the metric"
    )
    name: str = Field(title="Name", description="The name of the metric")
    code_snippet: str = Field(
        title="Code Snippet", description="The code snippet for the metric"
    )
    task_name: Optional[str] = Field(
        title="Task Name", description="The task name for the metric"
    )
    created_at: datetime.datetime
    updated_at: datetime.datetime


class ListMetricResponse(BaseModel):
    data: List[MetricResponse] = Field(
        title="Data", description="The list of metrics"
    )


class MetricRequest(BaseModel):
    name: str = Field(title="Name", description="The name of the metric")
    task: str = Field(
        title="Task Name", description="The task name for the metric"
    )
    code_snippet: str = Field(
        title="Code Snippet", description="The code snippet for the metric"
    )



from typing import Dict, Optional, Any

from pydantic import BaseModel, Field


class FeedbackQuery(BaseModel):
    task: str = Field(
        title="Task",
        description="The task for which feedback is being submitted",
    )
    generation_id: str = Field(
        title="Generation ID",
        description="The unique ID of the generation for which feedback is being submitted",
    )
    feedback_json: Dict[str, Any] = Field(
        title="Feedback JSON",
        description="The feedback JSON with one or many feedback items",
    )
    ground_truth: Optional[str] = Field(
        title="Ground Truth", description="The ground truth for the generation"
    )


class FeedbackResponse(BaseModel):
    task: str = Field(
        title="Task",
        description="The task for which feedback is being submitted",
    )
    generation_id: str = Field(
        title="Generation ID",
        description="The unique ID of the generation for which feedback is being submitted",
    )
    feedback: Dict[str, Any] = Field(
        title="Feedback JSON",
        description="The feedback JSON with one or many feedback items",
    )
    ground_truth: Optional[str] = Field(
        title="Ground Truth", description="The ground truth for the generation"
    )
    created_at: str = Field(
        title="Created At",
        description="The timestamp at which the feedback was created",
    )



from typing import Optional, List
from honeyhive.api.models.metrics import (
    MetricResponse,
    ListMetricResponse,
    MetricRequest,
)
from honeyhive.api.models.utils import DeleteResponse
from honeyhive.sdk.init import honeyhive_client


def get_metrics(
    project: str, metric_id: Optional[str]
) -> List[MetricResponse]:
    """Get all metrics"""
    client = honeyhive_client()
    return client.get_metrics(task=project, metric_id=metric_id)


def get_metric(metric_id: str) -> MetricResponse:
    """Get a metric"""
    client = honeyhive_client()
    return client.get_metric(metric_id=metric_id)


def create_metric(
    name: str, project: str, code_snippet: str
) -> MetricResponse:
    """Create a metric"""

    # TODO add validation for code_snippet

    client = honeyhive_client()
    return client.create_metric(
        metric=MetricRequest(
            name=name, task=project, code_snippet=code_snippet
        )
    )


def delete_metric(metric_id: str) -> DeleteResponse:
    """Delete a metric"""
    client = honeyhive_client()
    return client.delete_metric(metric_id=metric_id)


__all__ = ["get_metrics", "get_metric", "create_metric", "delete_metric"]



from typing import List, Optional, Any, Dict
from pydantic import BaseModel, Field
import datetime


class GenerateQuery(BaseModel):
    task: str = Field(
        title="Task",
        description="The task for which the generation is being requested",
    )
    input: Dict[str, str] = Field(
        title="Input", description="The input variables for the generation"
    )
    prompts: Optional[List[str]] = Field(
        title="Prompt IDs",
        description="The unique IDs of the prompts to generate from"
        + ". The prompts need to be associated with the task.",
    )
    model_id: Optional[str] = Field(
        title="Model ID",
        description="The unique ID of the fine-tuned model to generate from",
    )
    best_of: Optional[int] = Field(
        title="Best Of", description="The number of generations to be returned"
    )
    metric: Optional[str] = Field(
        title="Metric",
        description="The metric used for sorting the generations",
    )
    sampling: Optional[str] = Field(
        title="Sampling",
        description="The sampling method used for generating the generations"
        + " - works for open source models only. OpenAI defaults to nucleus sampling.",
    )
    user_properties: Optional[Dict[str, Any]] = Field(
        title="User Properties",
        description="The user properties to track for the generation",
    )
    source: Optional[str] = Field(
        title="Source", description="The source of the generation"
    )
    stream: Optional[bool] = Field(
        title="Streaming",
        description="Whether the generation is to be streamed",
    )


class GenerationResponse(BaseModel):
    generation_id: str = Field(
        title="Generation ID", description="The unique ID of the generation"
    )
    version: str = Field(
        title="Prompt ID", description="The unique ID of the prompt"
    )
    generation: str = Field(
        title="Completion", description="The generated completion"
    )


class ListGenerationResponse(BaseModel):
    task: str = Field(
        title="Task",
        description="The task for which the generation is being requested",
    )
    data: List[GenerationResponse] = Field(
        title="Generations", description="The list of generations"
    )
    created_at: str = Field(
        title="Created At",
        description="The timestamp at which the generations were created",
    )


class GenerationLoggingQuery(BaseModel):
    task: str = Field(
        title="Task",
        description="The task for which the generation is being requested",
    )
    model: str = Field(
        title="Model",
        description="The model that was used to generate the text",
    )
    hyperparameters: Dict[str, Any] = Field(
        title="Hyperparameters",
        description="The hyperparameters that were used to generate the text",
    )
    generation: str = Field(
        title="Generation",
        description="The text that was generated by your model provider",
    )
    prompt_template: Optional[str] = Field(
        title="Prompt Template",
        description="The prompt template that was used to generate the text",
    )
    inputs: Optional[Dict[str, Any]] = Field(
        title="Inputs", description="The inputs filled into the prompt"
    )
    prompt: Optional[str] = Field(
        title="Prompt", description="Prompt used to generate the text"
    )
    version: Optional[str] = Field(
        title="Version", description="The version of the prompt"
    )
    usage: Optional[Dict[str, Any]] = Field(
        title="Usage", description="The token usage for the generation"
    )
    source: Optional[str] = Field(
        title="Source", description="The source of the generation"
    )
    version: Optional[str] = Field(
        title="Version", description="The version of the prompt"
    )
    latency: Optional[float] = Field(
        title="Latency",
        description="The latency of the generation in milliseconds",
    )
    user_properties: Optional[Dict[str, Any]] = Field(
        title="User Properties",
        description="The user properties to track for the generation",
    )
    metadata: Optional[Dict[str, Any]] = Field(
        title="Metadata",
        description="The metadata to track for the generation",
    )
    ground_truth: Optional[str] = Field(
        title="Ground Truth", description="The ground truth for the generation"
    )
    feedback: Optional[Dict[str, Any]] = Field(
        title="Feedback",
        description="The feedback associated with this generation",
    )


class Generation(BaseModel):
    generation_id: str = Field(
        title="Generation ID", description="The unique ID of the generation"
    )
    version: str = Field(
        title="Prompt ID", description="The unique ID of the prompt"
    )
    task: str = Field(
        title="Task",
        description="The task for which the generation is being requested",
    )
    model: str = Field(
        title="Model",
        description="The model that was used to generate the text",
    )
    hyperparameters: Dict[str, Any] = Field(
        title="Hyperparameters",
        description="The hyperparameters that were used to generate the text",
    )
    generation: str = Field(
        title="Generation",
        description="The text that was generated by your model provider",
    )
    total_tokens: int = Field(
        title="Total Tokens",
        description="The total number of tokens generated",
    )
    completion_tokens: int = Field(
        title="Completion Tokens",
        description="The number of tokens generated for the completion",
    )
    cost: float = Field(title="Cost", description="The cost of the generation")
    latency: float = Field(
        title="Latency",
        description="The latency of the generation in milliseconds",
    )
    source: str = Field(
        title="Source", description="The source of the generation"
    )
    feedback: Optional[Dict[str, Any]] = Field(
        title="Feedback",
        description="The feedback associated with this generation",
    )



from typing import Dict, List, Optional, Any
from honeyhive.api.models.chat import (
    ChatQuery,
    ChatResponse
)
from honeyhive.sdk.init import honeyhive_client

def create(
    project: str,
    model: str = "gpt-3.5-turbo",
    messages: List[Dict[str, Any]] = [],
    provider: Optional[str] = "openai",
    version: Optional[str] = "unknown",
    inputs: Optional[Dict[str, Any]] = {},
    functions: Optional[List[Dict[str, Any]]] = None,
    function_call: Optional[str] = None,
    plugins: Optional[List[str]] = None,
    source: Optional[str] = "unknown",
    num_samples: Optional[int] = 1,
    stream: Optional[bool] = False,
    user: Optional[str] = None,
    metadata: Optional[Dict[str, Any]] = {},
    ground_truth: Optional[str] = "",
    **kwargs
):
    # parse the kwargs into hyperparameters
    hyperparameters = kwargs
    client = honeyhive_client()

    if stream == False:
        return client.chat(
            chat=ChatQuery(
                project=project,
                model=model,
                messages=messages,
                provider=provider,
                version=version,
                inputs=inputs,
                functions=functions,
                function_call=function_call,
                plugins=plugins,
                source=source,
                num_samples=num_samples,
                stream=stream,
                user=user,
                metadata=metadata,
                ground_truth=ground_truth,
                hyperparameters=hyperparameters,
            )
        )
    else:
        return client.chat_stream(
            chat=ChatQuery(
                project=project,
                model=model,
                messages=messages,
                provider=provider,
                version=version,
                inputs=inputs,
                functions=functions,
                function_call=function_call,
                plugins=plugins,
                source=source,
                num_samples=num_samples,
                stream=stream,
                user=user,
                metadata=metadata,
                ground_truth=ground_truth,
                hyperparameters=hyperparameters,
            )
        )

__all__ = ["create"]



from honeyhive.api.models.sessions import (
    SessionStartQuery,
    SessionStartResponse,
    SessionEndResponse,
    SessionEventQuery,
    SessionEventResponse,
    SessionQuery,
    SessionTrace,
    SessionFeedback
)

from typing import Dict, List, Any, Optional

import asyncio
import inspect
import traceback

# TODO add error handling

def start(
    project: str,
    source: str,
    session_name: str = None,
    session_type: str = "chain",
    user_properties: Dict = None,
    metadata: Dict = None
):
    """Start a session"""
    from honeyhive.sdk.init import honeyhive_client
    client = honeyhive_client()

    import uuid
    session_id = str(uuid.uuid4())

    if metadata is None:
        metadata = {}
    
    session_init = client.start_session(
        session=SessionStartQuery(
            session_id=session_id,
            project=project,
            source=source,
            session_name=session_name,
            session_type=session_type,
            user_properties=user_properties,
            metadata=metadata
        )
    )

    return session_init

def end(
    session_id: str
):
    """End a session"""
    from honeyhive.sdk.init import honeyhive_client
    client = honeyhive_client()

    client.end_session(
        session_id=session_id
    )

    return {
        "success": True
    }

def log_event(
    session_id: str,
    event_type: str,
    event_name: str,
    config: Dict,
    input: Dict,
    output: str,
    event_id: str = None,
    error: str = None,
    duration: float = None,
    children: List[Dict] = None,
    parent_id: str = None,
    event_properties: Dict = None,
    user_properties: Dict = None,
    metrics: Dict = None,
    feedback: Dict = None,
    metadata: Dict = None
):
    """Log an event"""
    from honeyhive.sdk.init import honeyhive_client
    client = honeyhive_client()

    if event_id is None:
        import uuid
        event_id = str(uuid.uuid4())
    
    client.log_event(
        session_id=session_id,
        session=SessionEventQuery(
            session_id=session_id,
            created_at="",
            event_id=event_id,
            event_type=event_type,
            event_name=event_name,
            config=config,
            inputs=input,
            output=output,
            error=error,
            duration=duration,
            children=children,
            parent_id=parent_id,
            event_properties=event_properties,
            user_properties=user_properties,
            metrics=metrics,
            feedback=feedback,
            metadata=metadata
        )
    )

    return {
        "success": True,
        "event_id": event_id
    }

def feedback(
    session_id: str,
    feedback: Dict,
    event_id: str = None,
    ground_truth: str = None
):
    """Log session feedback"""
    from honeyhive.sdk.init import honeyhive_client
    client = honeyhive_client()

    # TODO add logic to handle nesting feedback

    client.log_session_feedback(
        session_id=session_id,
        session=SessionFeedback(
            feedback=feedback,
            event_id=event_id,
            ground_truth=ground_truth
        )
    )

    return {
        "success": True
    }

def get(
    session_id: str
):
    """Get a session"""
    from honeyhive.sdk.init import honeyhive_client
    client = honeyhive_client()

    session = client.get_session(
        session_id=session_id
    )

    return session

def get_all(
    project: str,
    query: Dict,
    limit: int = None
):
    """Get all sessions"""
    from honeyhive.sdk.init import honeyhive_client
    client = honeyhive_client()

    sessions = client.get_sessions(
        query=SessionQuery(
            project=project,
            query=query,
            limit=limit
        )
    )

    return sessions

import traceback

def trace_chain(func):
  def wrapper(*args, **kwargs):
    stack = traceback.extract_stack()
    for frame_summary in stack:
      f = inspect.currentframe()
      print(f.f_locals)
    return func(*args, **kwargs)
  return wrapper


__all__ = [
    "start",
    "end",
    "log_event",
    "feedback",
    "trace_chain",
    "get",
    "get_all"
]


from typing import Dict, List, Optional, Any

def fill_template(
    template: str,
    inputs: Dict[str, str]
):
    """Fill a template with inputs"""
    import re
    import copy
    template = copy.deepcopy(template)
    for key, value in inputs.items():
        template = re.sub(r"{{\s*" + key + r"\s*}}", value, template)
    return template

def fill_chat_template(
    chat: List[Dict[str, str]],
    inputs: Dict[str, str]
):
    """Fill a chat template with inputs"""
    import copy
    chat = copy.deepcopy(chat)
    for message in chat:
        message["content"] = fill_template(message["content"], inputs)
    return chat

def calculate_openai_cost(
    model: str,
    usage: Dict[str, Any]
):
    """Calculate the cost of an OpenAI model"""
    openai_costs = {
        "ada": 0.0004,
        "babbage": 0.0005,
        "curie": 0.002,
        "davinci": 0.02,
        "turbo": 0.002
    }

    openai_costs_input = {
        "turbo": 0.0015,
        "turbo-16k": 0.003,
        "4": 0.03,
        "4-32k": 0.06
    }

    openai_costs_output = {
        "turbo": 0.002,
        "turbo-16k": 0.004,
        "4": 0.06,
        "4-32k": 0.12
    }

    cost = 0

    if not model:
        return 0
    
    if "gpt-4" in model:
        if "32k" in model:
            cost = openai_costs_input["4-32k"] * usage["prompt_tokens"]/1000 + openai_costs_output["4-32k"] * usage["completion_tokens"]/1000
        else:
            cost = openai_costs_input["4"] * usage["prompt_tokens"]/1000 + openai_costs_output["4"] * usage["completion_tokens"]/1000
    elif "gpt-3.5-turbo" in model:
        if "16k" in model:
            cost = openai_costs_input["turbo-16k"] * usage["prompt_tokens"]/1000 + openai_costs_output["turbo-16k"] * usage["completion_tokens"]/1000
        else:
            cost = openai_costs_input["turbo"] * usage["prompt_tokens"]/1000 + openai_costs_output["turbo"] * usage["completion_tokens"]/1000
    else:
        for key in openai_costs:
            if key in model:
                cost = openai_costs[key] * usage["total_tokens"]/1000
                break

    # approximate cost to 5 decimal places
    cost = cost * 100000
    cost = int(cost)
    cost = cost / 100000

    return cost

__all__ = [
    "fill_template",
    "fill_chat_template",
    "calculate_openai_cost"
]


from typing import Optional, List
from honeyhive.api.models.tasks import (
    TaskCreationQuery,
    TaskUpdateQuery,
    TaskResponse,
    ListTaskResponse,
)
from honeyhive.api.models.fine_tuned_models import FineTunedModelResponse
from honeyhive.api.models.prompts import PromptResponse
from honeyhive.api.models.datasets import DatasetResponse
from honeyhive.api.models.metrics import MetricResponse
from honeyhive.api.models.utils import DeleteResponse
from honeyhive.sdk.init import honeyhive_client


def get_projects(
    name: Optional[str] = None,
) -> ListTaskResponse:
    """Get all tasks"""
    client = honeyhive_client()
    return client.get_tasks(name=name)


def get(name: str) -> TaskResponse:
    """Get a task"""
    client = honeyhive_client()
    return client.get_tasks(name=name)


def create(
    name: str,
    type: str,
    fine_tuned_models: Optional[List[FineTunedModelResponse]] = [],
    prompts: Optional[List[PromptResponse]] = [],
    datasets: Optional[List[DatasetResponse]] = [],
    metrics: Optional[List[MetricResponse]] = []
) -> TaskResponse:
    """Create a task"""
    client = honeyhive_client()
    return client.create_task(
        task=TaskCreationQuery(
            name=name,
            type=type,
            fine_tuned_models=fine_tuned_models,
            prompts=prompts,
            datasets=datasets,
            metrics=metrics,
        )
    )


def update(
    task_id: str,
    name: Optional[str] = None,
    type: Optional[str] = None,
    fine_tuned_models: Optional[List[FineTunedModelResponse]] = [],
    prompts: Optional[List[PromptResponse]] = [],
    datasets: Optional[List[DatasetResponse]] = [],
    metrics: Optional[List[MetricResponse]] = [],
) -> TaskResponse:
    """Update a task"""
    client = honeyhive_client()
    return client.update_task(
        task_id=task_id,
        task=TaskUpdateQuery(
            name=name,
            type=type,
            fine_tuned_models=fine_tuned_models,
            prompts=prompts,
            datasets=datasets,
            metrics=metrics,
        ),
    )


def delete(name: str) -> DeleteResponse:
    """Delete a task"""
    client = honeyhive_client()
    return client.delete_task(name=name)


__all__ = [
    "get_projects",
    "get",
    "create",
    "update",
    "delete",
]



from .metrics import *
from .datasets import *
from .prompts import *
from .projects import *
from .fine_tuned_models import *
from .generations import *
from .Completion import *
from .ChatCompletion import *
from .feedback import *
from .utils import *
from .evaluations import *
from .sessions import *
from .tracer import *
from .langchain_tracer import *
from .llamaindex_tracer import *
from .chat import *

import os
from typing import Optional

api_key = os.environ.get("HONEYHIVE_API_KEY")
# Path of a file with an API key, whose contents can change. Supercedes
# `api_key` if set.  The main use case is volume-mounted Kubernetes secrets,
# which are updated automatically.
api_key_path: Optional[str] = os.environ.get("HONEYHIVE_API_KEY_PATH")

organization = os.environ.get("HONEYHIVE_ORGANIZATION")
api_base = os.environ.get(
    "HONEYHIVE_API_BASE", "https://api.honeyhive.ai/"
)  # "https://api.honeyhive.ai/v1")

openai_api_key = os.environ.get("OPENAI_API_KEY")
cohere_api_key = os.environ.get("COHERE_API_KEY")
huggingface_api_key = os.environ.get("HUGGINGFACE_API_KEY")

__all__ = [
    "metrics",
    "datasets",
    "prompts",
    "projects",
    "chat",
    "fine_tuned_models",
    "Completion",
    "ChatCompletion",
    "generations",
    "feedback",
    "evaluations",
    "sessions",
    "tracer",
    "langchain_tracer",
    "llamaindex_tracer",
    "utils",
    "api_key",
    "api_key_path",
    "organization",
    "api_base",
    "openai_api_key",
    "cohere_api_key",
    "huggingface_api_key",
]



from typing import Dict, List, Optional, Any
from honeyhive.api.models.generations import (
    GenerationResponse,
    GenerationLoggingQuery,
)
from honeyhive.sdk.init import honeyhive_client

import honeyhive


def create(
    project: str,
    model: str,
    prompt: str or List[str],
    source: Optional[str] = None,
    version: Optional[str] = None,
    engine: Optional[str] = None,
    max_tokens: Optional[int] = 16,
    temperature: Optional[float] = 1.0,
    top_p: Optional[float] = 1.0,
    suffix: Optional[str] = None,
    presence_penalty: Optional[float] = 0.0,
    frequency_penalty: Optional[float] = 0.0,
    n: Optional[int] = 1,
    stream: Optional[bool] = False,
    echo: Optional[bool] = False,
    logprobs: Optional[Any] = None,
    stop: Optional[str] = None,
    logit_bias: Optional[Dict[str, float]] = None,
    user: Optional[str] = None,
    best_of: Optional[int] = None,
) -> GenerationResponse:
    """Generate completions"""
    # send all the specified args to openai's completion endpoint
    import openai
    import os
    import time

    openai.api_key = os.environ.get("OPENAI_API_KEY", honeyhive.openai_api_key)

    if engine is not None:
        model = engine

    args_for_openai = {
        "model": model,
        "prompt": prompt,
        "max_tokens": max_tokens,
        "temperature": temperature,
        "top_p": top_p,
        "sufix": suffix,
        "presence_penalty": presence_penalty,
        "frequency_penalty": frequency_penalty,
        "n": n,
        "stream": stream,
        "echo": echo,
        "logprobs": logprobs,
        "stop": stop,
        "logit_bias": logit_bias,
        "user": user,
        "best_of": best_of,
    }

    # drop all the args which have None values
    args_for_openai = {
        k: v for k, v in args_for_openai.items() if v is not None
    }

    start = time.time()
    response = openai.Completion.create(**args_for_openai)
    end = time.time()

    latency = (end - start) * 1000

    # log generation via honeyhive
    client = honeyhive_client()

    # from args for openai, get the args for honeyhive
    # remove the args which are not in honeyhive's logging query
    args_for_honeyhive = args_for_openai.copy()
    args_for_honeyhive.pop("model")
    args_for_honeyhive.pop("prompt")
    # pop user if present
    args_for_honeyhive.pop("user", None)

    honeyhive_response = client.log_generation(
        GenerationLoggingQuery(
            task=project,
            model=model,
            prompt=prompt,
            hyperparameters=args_for_honeyhive,
            generation=response.choices[0].text,
            usage=response.usage,
            latency=latency,
            source=source,
            version=version,
        )
    )
    generation_id = honeyhive_response.generation_id

    # add generation_id to openai's response
    response.generation_id = generation_id

    return response


__all__ = ["create"]



import requests
from requests import HTTPError
from uplink import Body, Consumer, Query, delete, get, json
from uplink import put, post, response_handler, returns
from uplink.auth import BearerToken
from typing import List, Dict, Any

from honeyhive.api.models.utils import DeleteResponse

from honeyhive.api.models.tasks import (
    TaskCreationQuery,
    ListTaskResponse,
    TaskResponse,
    TaskUpdateQuery
)

from honeyhive.api.models.evaluations import (
    EvaluationLoggingQuery,
    SuccessResponse
)

from honeyhive.api.models.prompts import (
    ListPromptResponse,
    PromptResponse,
    PromptCreationQuery,
    PromptUpdateQuery
)

from honeyhive.api.models.fine_tuned_models import (
    ListFineTunedModelResponse,
    FineTunedModelResponse
)

from honeyhive.api.models.datasets import (
    ListDatasetResponse,
    DatasetResponse,
    FetchDataset,
    UploadDataset
)

from honeyhive.api.models.generations import (
    ListGenerationResponse,
    GenerationResponse,
    GenerateQuery,
    GenerationLoggingQuery,
    Generation
)

from honeyhive.api.models.chat import (
    ChatQuery,
    ChatResponse
)

from honeyhive.api.models.feedback import (
    FeedbackQuery,
    FeedbackResponse
)

from honeyhive.api.models.metrics import (
    ListMetricResponse,
    MetricResponse,
    MetricRequest
)

from honeyhive.api.models.sessions import (
    SessionStartQuery,
    SessionStartResponse,
    SessionEndResponse,
    SessionEventQuery,
    SessionEventResponse,
    SessionQuery,
    SessionTrace,
    SessionFeedback
)


def raise_for_status(response: requests.Response):
    """Checks whether the response was successful."""
    try:
        response.raise_for_status()
        return response
    finally:
        if response.status_code >= 400:
            raise HTTPError(response.text)


@response_handler(raise_for_status)
class HoneyHive(Consumer):
    """HoneyHive API Client"""

    # Tasks API
    @returns.json
    @get("/tasks", args={"name": Body})
    def get_tasks(
        self,
        name: Body(type=str) = None,
    ) -> List[TaskResponse]:
        """Get all tasks"""
    
    @json
    @returns.json
    @post("/tasks", args={"task": Body(TaskCreationQuery)})
    def create_task(
        self,
        task: Body(TaskCreationQuery),
    ) -> TaskResponse:
        """Create a task"""

    @json
    @returns.json
    @put("/tasks/{id}", args={"task": Body(TaskUpdateQuery)})
    def update_task(
        self,
        id: str,
        task: Body(TaskUpdateQuery),
    ) -> TaskResponse:
        """Update a task"""
    
    @returns.json
    @delete("/tasks/{name}")
    def delete_task(
        self,
        name: str,
    ) -> DeleteResponse:
        """Delete a task"""
    
    @returns.json
    @get("/generations", args={"task": Query, "prompt": Query, "model_id": Query})
    def get_generations(
        self,
        task: Query = None,
        prompt: Query = None,
        model_id: Query = None
    ) -> List[Generation]:
        """Get all generations"""
    
    # Prompts API
    @returns.json
    @get("/prompts", args={"task": Query, "name": Query})
    def get_prompts(
        self,
        task: Query = None,
        name: Query = None
    ) -> List[PromptResponse]:
        """Get all prompts"""

    @json
    @returns.json
    @post("/prompts", args={"prompt": Body(PromptCreationQuery)})
    def create_prompt(
        self,
        prompt: Body(PromptCreationQuery),
    ) -> PromptResponse:
        """Create a prompt"""
    
    @returns.json
    @put("/prompts/{id}", args={"prompt": Body(PromptUpdateQuery)})
    def update_prompt(
        self,
        id: str,
        prompt: Body(PromptUpdateQuery),
    ) -> PromptResponse:
        """Update a prompt"""
    
    @returns.json
    @delete("/prompts/{name}")
    def delete_prompt(
        self,
        name: str,
    ) -> DeleteResponse:
        """Delete a prompt"""
    

    # Fine Tuned Models API
    @returns.json
    @get("/fine_tuned_models", args={"task": Query, "model_id": Query})
    def get_fine_tuned_models(
        self,
        task: Query = Query(None),
        model_id: Query = Query(None),
    ) -> List[FineTunedModelResponse]:
        """Get all fine tuned models"""
    
    @returns.json
    @get("/fine_tuned_models/{id}")
    def get_fine_tuned_model(
        self,
        id: str,
    ) -> FineTunedModelResponse:
        """Get a fine tuned model"""
    
    # Datasets API
    @returns.json
    @get("/datasets", args={"task": Query, "prompt": Query, "name": Query, "purpose": Query})
    def get_datasets(
        self,
        task: Query = None,
        prompt: Query = None,
        name: Query = None,
        purpose: Query = None,
    ) -> List[DatasetResponse]:
        """Get all datasets"""
    
    @json
    @returns.json
    @post("/datasets", args={"dataset": Body(type=UploadDataset)})
    def create_dataset(
        self,
        dataset: Body(type=UploadDataset),
    ) -> DatasetResponse:
        """Create a dataset"""
    
    @returns.json
    @get("/datasets/{name}")
    def get_dataset(
        self,
        name: str,
    ) -> DatasetResponse:
        """Get a dataset"""
    
    @returns.json
    @delete("/datasets/{name}")
    def delete_dataset(
        self,
        name: str,
    ) -> DeleteResponse:
        """Delete a dataset"""
    
    # Metrics API
    @returns.json
    @get("/metrics", args={"task": Query, "metric_id": Query})
    def get_metrics(
        self,
        task: Query = Query(None),
        metric_id: Query = Query(None),
    ) -> List[MetricResponse]:
        """Get all metrics"""

    @returns.json
    @post("/metrics", args={"metric": Body(MetricRequest)})
    def create_metric(
        self,
        metric: Body(MetricRequest),
    ) -> MetricResponse:
        """Create a metric"""
    
    @returns.json
    @get("/metrics/{id}")
    def get_metric(
        self,
        id: str,
    ) -> MetricResponse:
        """Get a metric"""
    
    @returns.json
    @delete("/metrics/{id}")
    def delete_metric(
        self,
        id: str,
    ) -> DeleteResponse:
        """Delete a metric"""
    
    # Generations API
    @json
    @returns.json
    @post("/generations", args={"generation": Body(GenerateQuery)})
    def generate(
        self,
        generation: Body(GenerateQuery),
    ) -> GenerationResponse:
        """Generate a text"""
    
    # Generations API stream
    @json
    @post("/generations", args={"generation": Body(GenerateQuery)})
    def generate_stream(
        self,
        generation: Body(GenerateQuery),
    ):
        """Generate a text"""
    
    # Chat API
    @json
    @returns.json
    @post("/chat", args={"chat": Body(ChatQuery)})
    def chat(
        self,
        chat: Body(ChatQuery),
    ) -> ChatResponse:
        """Generate a chat completion"""

    # Chat API
    @json
    @post("/chat", args={"chat": Body(ChatQuery)})
    def chat_stream(
        self,
        chat: Body(ChatQuery),
    ):
        """Generate a chat completion stream"""
    
    # Log Generation
    @json
    @returns.json
    @post("/generations/log", args={"generation": Body(GenerationLoggingQuery)})
    def log_generation(
        self,
        generation: Body(GenerationLoggingQuery),
    ) -> GenerationResponse:
        """Generate a text"""
    
    # Feedback API
    @json
    @returns.json
    @post("/feedback", args={"feedback": Body(FeedbackQuery)})
    def feedback(
        self,
        feedback: Body(FeedbackQuery),
    ) -> FeedbackResponse:
        """Send feedback"""
    
    # Evaluations API
    @json
    @returns.json
    @post("/evaluations", args={"evaluation": Body(EvaluationLoggingQuery)})
    def log_evaluation(
        self,
        evaluation: Body(EvaluationLoggingQuery),
    ) -> SuccessResponse:
        """Log an evaluation"""
    
    # Sessions API
    @json
    @returns.json
    @post("/session/start", args={"session": Body(SessionStartQuery)})
    def start_session(
        self,
        session: Body(SessionStartQuery),
    ) -> SessionStartResponse:
        """Start a session"""
    
    @json
    @returns.json
    @post("/session/{session_id}/end")
    def end_session(
        self,
        session_id: str,
    ) -> SessionEndResponse:
        """End a session"""
    
    @json
    @returns.json
    @post("/session/{session_id}/event", args={"session": Body(SessionEventQuery)})
    def log_event(
        self,
        session_id: str,
        session: Body(SessionEventQuery),
    ) -> SessionEventResponse:
        """Log an event"""
    
    @json
    @returns.json
    @post("/session/{session_id}/feedback", args={"session": Body(SessionFeedback)})
    def log_session_feedback(
        self,
        session_id: str,
        session: Body(SessionFeedback),
    ) -> SuccessResponse:
        """Log session feedback"""
    
    @json
    @returns.json
    @get("/session/{session_id}")
    def get_session(
        self,
        session_id: str,
    ) -> SessionEventQuery:
        """Get a session"""
    
    @json
    @returns.json
    @get("/session", args={"query": Body(SessionQuery)})
    def get_sessions(
        self,
        query: SessionQuery,
    ) -> List[SessionEventQuery]:
        """Get all sessions"""
    
    @json
    @returns.json
    @post("/sessions/{session_id}/traces", args={"trace": Body(SessionTrace)})
    def log_trace(
        self,
        session_id: str,
        trace: Body(SessionTrace),
    ) -> SuccessResponse:
        """Log a trace"""

def get_client(base_url: str, api_key: str) -> HoneyHive:
    """Get a HoneyHive API client"""
    return HoneyHive(base_url=base_url, auth=BearerToken(api_key))



from typing import Any, Dict, List, Optional
from honeyhive.api.models.feedback import FeedbackQuery, FeedbackResponse
from honeyhive.sdk.init import honeyhive_client


def generation_feedback(
    project: str,
    generation_id: str,
    feedback_json: Dict[str, Any],
    ground_truth: Optional[str] = None,
) -> FeedbackResponse:
    """Submit feedback"""
    client = honeyhive_client()
    return client.feedback(
        feedback=FeedbackQuery(
            task=project,
            generation_id=generation_id,
            feedback_json=feedback_json,
            ground_truth=ground_truth,
        )
    )


__all__ = ["generation_feedback"]



#!/usr/bin/env python

"""
distutils/setuptools install script.
"""

from setuptools import setup, find_packages

package_data = {'': ['*']}

requires = [
    'requests>=2.25.1',
    'pydantic>=1.8.2',
    'uplink>=0.0.2',
    'pandas>=1.3.2',
    'openai>=0.27.8',
    'langchain>=0.0.251',
    'llama_index>=0.8.13'
]

entry_points = {'console_scripts': ['honeyhive = honeyhive.cli:honeyhive']}

setup(
    name='honeyhive',
    version='0.1.31',
    description='The HoneyHive SDK for Python',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='HoneyHive',
    author_email="support@honeyhive.ai",
    scripts=[],
    url='https://github.com/codehruv/honeyhive-sdk',
    packages=find_packages(exclude=['tests*']),
    package_data=package_data,
    include_package_data=True,
    install_requires=requires,
    license="Apache License 2.0",
    python_requires=">= 3.7",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
    project_urls={
        'Documentation': 'https://docs.honeyhive.ai/',
        'Source': 'https://github.com/codehruv/honeyhive-sdk',
    },
    entry_points=entry_points,
)



from typing import Optional, List, Dict, Any
from pydantic import BaseModel, Field
import datetime


class DatasetResponse(BaseModel):
    name: str = Field(
        title="Dataset Name", description="The name of the dataset file"
    )
    bytes: int = Field(
        title="Bytes", description="The size of the dataset in bytes"
    )
    created_at: datetime.datetime = Field(
        title="Created At",
        description="The date and time when the dataset was created",
    )
    task: str = Field(
        title="Task", description="The task related to the dataset"
    )
    prompt: Optional[str] = Field(
        title="Prompt", description="The prompt related to the dataset"
    )
    purpose: str = Field(
        title="Purpose", description="The purpose of the dataset"
    )
    description: Optional[str] = Field(
        title="Description", description="The description of the dataset"
    )


class ListDatasetResponse(BaseModel):
    data: List[DatasetResponse] = Field(
        title="Data", description="The list of datasets"
    )


class FetchDataset(BaseModel):
    name: Optional[str] = Field(
        title="Dataset ID", description="The unique ID of the dataset"
    )
    task: Optional[str] = Field(
        title="Task", description="The dataset related to a task"
    )
    prompt: Optional[str] = Field(
        title="Prompt", description="The dataset related to a prompt"
    )
    purpose: Optional[str] = Field(
        title="Purpose", description="The purpose of the dataset"
    )


class UploadDataset(BaseModel):
    name: str = Field(title="Name", description="The name of the dataset")
    task: str = Field(
        title="Task", description="The dataset related to a task"
    )
    prompt: Optional[str] = Field(
        title="Prompt", description="The dataset related to a prompt"
    )
    purpose: str = Field(
        title="Purpose", description="The purpose of the dataset"
    )
    description: Optional[str] = Field(
        title="Description", description="The description of the dataset"
    )
    file: List[Dict[str, Any]] = Field(
        title="File", description="The dataset file"
    )



import honeyhive

honeyhive.api_key = "YOUR_API_KEY"
honeyhive.openai.api_key = "OPENAI_API_KEY"


####################
# Generation and feedback examples
####################

# Create a task
task = honeyhive.create_task(
    task="Blog writer",
    description="Writing blog posts for users based on topic and tone",
)

# Create a prompt
prompt1 = honeyhive.create_prompt(
    task="Blog title writer",
    name="Blog title writer 1",
    description="First attempt at a blog title writer",
    prompt="Write a blog post about {topic} in the {tone} tone.",
    model="openai:text-davinci-003",
    hyperparameters={"max_tokens": 100, "temperature": 0.9},
    few_shot_examples=[
        {
            "topic": "weather",
            "tone": "informative",
            "completion": "Temperature is expected to be 90 degrees today.",
        },
        {
            "topic": "weather",
            "tone": "funny",
            "completion": "Going to the beach today? Don't forget your sunscreen!",
        },
    ],
)

# Call generations
generations = honeyhive.generate(
    task="Blog title writer",
    inputs=[
        {"topic": "weather", "tone": "informative"},
        {"topic": "weather", "tone": "funny"},
    ],
    # prompts=["Blog title writer 1"],
    best_of=2,
    metrics="LexicalDiversity",
)

# Proivde feedback on generations
honeyhive.feedback(
    generation_id=generations[0].generation_id,
    feedback={
        "user": "user1",
        "user_country": "USA",
        "rejected": False,
        "edited": True,
        "feedback": "This is a great blog post!",
        "final_version": "Temperature rises up to 90 degrees today.",
    },
)

# Look at generations
honeyhive.get_generations(
    task="Blog title writer",
    prompts=["Blog title writer 1", "Blog title writer 2"],
    start_date="2022-12-01",
    end_date="2021-12-03",
)

####################
# Evaluation examples
####################

# Upload a validation dataset to test over
dataset = honeyhive.create_dataset(
    task="Blog title writer",
    name="blog_title_test",
    description="Validation dataset for blog title writer",
    purpose="validation",
    data=open("blog_title_test.csv", "r").read(),
)

# Add a metric to test over
metric = honeyhive.create_metric(
    task="Blog title writer",
    name="IsPassive",
    code_snippet="""
    def is_passive_voice(sentence):
        import nltk
        from nltk import word_tokenize, pos_tag
        from nltk.corpus import wordnet as wn
        from nltk.stem import WordNetLemmatizer
        lemmatizer = WordNetLemmatizer()
        passive = False
        # tokenize sentence
        tokens = word_tokenize(sentence)
        # tag sentence
        tagged = pos_tag(tokens)
        # check if verb is in passive form
        for i in range(len(tagged)):
            if tagged[i][1] == 'VBN':
                # check if verb is in passive form
                if tagged[i][0] == lemmatizer.lemmatize(tagged[i][0], wn.VERB):
                    passive = True
        # map True/False to 1/0
        return int(passive)
    """,
)

# import prompts needed for the task
blog_title_prompts = honeyhive.get_prompts(task="Blog title writer")

# run evaluation for the prompts on the dataset
test_results = honeyhive.evaluate(
    task="Blog title writer",
    prompts=[prompt.name for prompt in blog_title_prompts],
    dataset="blog_title_validation",
    metrics=["BLEU", "ROGUE", "METEOR", "IsPassive"],
)

print(test_results)

# output looks like:
# [
#   {
#     "prompt": "Blog title writer",
#     "dataset": "blog_title_validation",
#     "metrics": {
#       "BLEU": 0.5,
#       "ROGUE": 0.5,
#       "METEOR": 0.5,
#       "IsPassive": 0.5
#     }
#   },
#   ...
# ]

##################
# Fine tuning examples
##################

# get all generations for a task
generations = honeyhive.Completion(
    task="financial-statement-summarizer",
    start_date="2022-12-01",
    end_date="2022-12-31",
)

# filter generations where passive is true, user_country is US, lexical diversity is greater than 0.5
generations_df = pd.DataFrame(generations)
generations_df = generations_df[
    (generations_df["feedback"]["passive"] == True)
    & (generations_df["feedback"]["user_country"] == "US")
    & (generations_df["feedback"]["lexical_diversity"] > 0.5)
    & (generations_df["feedback"]["BLEU"] > 0.5)
]
generations_df = generations_df[["prompt", "generation"]]

# fine tune model on the filtered generations
fine_tuning_job = honeyhive.fine_tune(
    task="financial-statement-summarizer",
    generations=generations_df.to_dict("records"),
    model="openai:text-curie-001",
    hyperparameters={
        "learning_rate": 1e-5,
        "batch_size": 8,
        "epochs": 1,
        "max_length": 512,
    },
    strategy="SFT",
)

print(fine_tuning_job)
# output looks like:
# {
#   "job_id": "job_id",
#   "status": "running",
#   "task": "financial-statement-summarizer",
#   "model": "openai:text-curie-001",
#   "hyperparameters": {
#     "learning_rate": 1e-5,
#     "batch_size": 8,
#     "epochs": 1,
#     "max_length": 512
#   }
# }
